package com.chinaitop.depot.job.service.impl;

import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.apache.commons.lang3.StringUtils;
import org.quartz.CronExpression;
import org.quartz.CronTrigger;
import org.quartz.Scheduler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.chinaitop.depot.job.mapper.ScheduleJobMapper;
import com.chinaitop.depot.job.model.ScheduleJob;
import com.chinaitop.depot.job.model.ScheduleJobExample;
import com.chinaitop.depot.job.model.ScheduleJobExample.Criteria;
import com.chinaitop.depot.job.service.ScheduleJobService;
import com.chinaitop.depot.job.utils.Constant.ScheduleStatus;
import com.chinaitop.depot.job.utils.ScheduleUtils;
import com.chinaitop.depot.job.utils.SpringContextUtils;
import com.github.pagehelper.PageHelper;

@Service("scheduleJobService")
public class ScheduleJobServiceImpl implements ScheduleJobService {
	@Autowired
    private Scheduler scheduler;
	@Autowired
	private ScheduleJobMapper scheduleJobMapper;
	
	/**
	 * 项目启动时，初始化定时器
	 */
	@PostConstruct
	public void init() {
	    ScheduleJobExample scheduleJobExample = new ScheduleJobExample();
		List<ScheduleJob> scheduleJobList = scheduleJobMapper.selectByExample(scheduleJobExample);
		for(ScheduleJob scheduleJob : scheduleJobList){
			CronTrigger cronTrigger = ScheduleUtils.getCronTrigger(scheduler, scheduleJob.getId());
            //如果不存在，则创建
            if(cronTrigger == null) {
                ScheduleUtils.createScheduleJob(scheduler, scheduleJob);
            }else {
                ScheduleUtils.updateScheduleJob(scheduler, scheduleJob);
            }
		}
	}
	
	@Override
	public ScheduleJob find(Long id) {
		return scheduleJobMapper.selectByPrimaryKey(id);
	}

	/*@Override
	public int queryTotal(Map<String, Object> map) {
		return scheduleJobMapper.queryTotal(map);
	}*/

	@Override
	@Transactional
	public Map<String, String> add(ScheduleJob scheduleJob, Integer orgId, Integer userId) {
	    Map<String, String> checkMap = check(scheduleJob);
	    if (checkMap != null) {
	        return checkMap;
	    }
	    
	    Map<String, String> returnMap = new HashMap<String, String>();
		scheduleJob.setCreateTime(new Date());
		scheduleJob.setStatus((byte)ScheduleStatus.NORMAL.getValue());
		scheduleJob.setOrgId(orgId);
		scheduleJob.setCreaterId(userId);
        scheduleJobMapper.insert(scheduleJob);
        
        ScheduleUtils.createScheduleJob(scheduler, scheduleJob);
        returnMap.put("status", "success");
        return returnMap;
    }
	
	// 校验 cron表达式，类是否存在，方法是否存在.
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private Map<String, String> check(ScheduleJob scheduleJob) {
	    Map<String, String> returnMap = new HashMap<String, String>();
	    if (StringUtils.isBlank(scheduleJob.getCronExpression())) {
            returnMap.put("status", "error");
            returnMap.put("msg", "cron表达式不能为空！");
            return returnMap;
        } else {
            Boolean isEffective =  checkCron(scheduleJob.getCronExpression());
            if (!isEffective) {
                returnMap.put("status", "error");
                returnMap.put("msg", "cron表达式不合法！");
                return returnMap;
            }
        }
        
        
        if (StringUtils.isBlank(scheduleJob.getBeanName())) {
            returnMap.put("status", "error");
            returnMap.put("msg", "bean名称不能为空！");
            return returnMap;
        } else {
            try {
                Object bean = SpringContextUtils.getBean(scheduleJob.getBeanName());
                if (bean == null) {
                    returnMap.put("status", "error");
                    returnMap.put("msg", "不存在此bean！");
                    return returnMap;
                } else {
                    // 先判断方法名称.
                    // 然后通过反射判断该类是否存在此方法.
                    if (StringUtils.isBlank(scheduleJob.getMethodName())) {
                        returnMap.put("status", "error");
                        returnMap.put("msg", "方法名称不能为空！");
                        return returnMap;
                    } else {
                        Class clazz = bean.getClass();
                        try {
                            clazz.getMethod(scheduleJob.getMethodName());
                        } catch (Exception e) {
                            e.printStackTrace();
                            returnMap.put("status", "error");
                            returnMap.put("msg", "该类不存在此方法名称的方法！");
                            return returnMap;
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                returnMap.put("status", "error");
                returnMap.put("msg", "不存在此bean！");
                return returnMap;
            }
        }
        return null;
	}
	
	
	/**
	 * 检测cron表达式是否合法.
     * @param cronExpression
     * @return
     */
    private Boolean checkCron(String cron) {
        try {
            new CronExpression(cron);
        } catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
	@Transactional
	public Map<String, String> update(ScheduleJob scheduleJob, Integer userId) {
        Map<String, String> checkMap = check(scheduleJob);
        if (checkMap != null) {
            return checkMap;
        }
        
        Map<String, String> returnMap = new HashMap<String, String>();
        ScheduleUtils.updateScheduleJob(scheduler, scheduleJob);
        scheduleJob.setUpdaterId(userId);
        scheduleJob.setUpdateTime(new Date());
        scheduleJobMapper.updateByPrimaryKey(scheduleJob);
        returnMap.put("status", "success");
        return returnMap;
    }

	@Override
	@Transactional
    public void remove(Long id) {
		ScheduleUtils.deleteScheduleJob(scheduler, id);
		//删除数据
		scheduleJobMapper.deleteByPrimaryKey(id);
	}

	@Override
    public int updateBatch(Long[] ids, int status){
    	Map<String, Object> map = new HashMap<>();
    	map.put("list", ids);
    	map.put("status", status);
    	return scheduleJobMapper.updateBatch(map);
    }
    
	@Override
	@Transactional
    public void run(Long id) {
		ScheduleUtils.run(scheduler, find(id));
    }

	@Override
	@Transactional
    public void pause(Long id) {
	    ScheduleUtils.pauseJob(scheduler, id);
	    
	    ScheduleJob scheduleJob = new ScheduleJob();
	    scheduleJob.setId(id);
        scheduleJob.setUpdateTime(new Date());
        scheduleJob.setStatus((byte) ScheduleStatus.PAUSE.getValue());
        scheduleJobMapper.updateByPrimaryKeySelective(scheduleJob);
    }

	@Override
	@Transactional
    public void resume(Long id) {
		ScheduleUtils.resumeJob(scheduler, id);
		
	    ScheduleJob scheduleJob = new ScheduleJob();
        scheduleJob.setId(id);
        scheduleJob.setUpdateTime(new Date());
        scheduleJob.setStatus((byte) ScheduleStatus.NORMAL.getValue());
        scheduleJobMapper.updateByPrimaryKeySelective(scheduleJob);
    }

    @Override
    public List<ScheduleJob> queryByExample(Integer pageNum, Integer pageSize, Integer orgId) {
        ScheduleJobExample scheduleJobExample = new ScheduleJobExample();
        Criteria criteria = scheduleJobExample.createCriteria();
        criteria.andOrgIdEqualTo(orgId);
        PageHelper.startPage(pageNum, pageSize);
        return scheduleJobMapper.selectByExample(scheduleJobExample);
    }

    
}
